{-
Author: Ken Lambert

File: Generator.hs

This module defines a grammar and a vocabulary for generating random sentences

To generate a single sentence, call sentence within a monadic context or in ghci.

-}

module Generator where

import MyRandom (randomInt, pickRandom)

-- Vocabulary consists of nouns, verbs, prepositions, and articles

nouns :: [String]
nouns = ["bat", "boy", "girl", "dog", "cat", "chair",
         "fence", "table", "computer", "cake", "field"]

verbs :: [String]
verbs = ["hit", "threw", "pushed", "ate", "dragged", "jumped"]

prepositions :: [String]
prepositions = ["with", "to", "from", "on", "below",
                "above", "beside"]

articles :: [String]
articles = ["a", "the"]

-- Syntax of sentences and other phrases

-- sentence = nounphrase verbphrase
sentence :: IO String
sentence = do
    np <- nounPhrase 
    vp <- verbPhrase
    return (np ++ " " ++ vp)

-- nounphrase = article noun
nounPhrase :: IO String
nounPhrase = do
    article <- pickRandom articles 
    noun <- pickRandom nouns
    return (article ++ " " ++ noun)

-- verbphrase = verb nounphrase prepositionalphrase
verbPhrase :: IO String
verbPhrase = do
    verb <-pickRandom verbs 
    np <- nounPhrase 
    pp <- prepositionalPhrase
    return (verb ++ " " ++ np ++ " " ++ pp)

-- prepositionalphrase = preposition nounphrase
prepositionalPhrase :: IO String
prepositionalPhrase = do
    prep <- pickRandom prepositions 
    np <- nounPhrase
    return (prep ++ " " ++ np)
    
testGenerator :: Int -> IO ()
testGenerator iterations = do
    if iterations == 0 then
        return ()
    else do
        s <- sentence
        putStrLn s
        testGenerator (iterations - 1)

